@@ -237,20 +237,23 @@ hildon_desktop_panel_expandable_init (HildonDesktopPanelExpandable *panel)
 static void 
 hildon_desktop_panel_expandable_hide_extension (HildonDesktopPanelExpandable *panel)
 {
-  gdk_pointer_ungrab (GDK_CURRENT_TIME);
-  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+  if (panel->priv->extension_opened)
+  {
+    gdk_pointer_ungrab (GDK_CURRENT_TIME);
+    gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 
-  if (!HILDON_DESKTOP_IS_PANEL_EXPANDABLE (panel))
-    return;
-	  
-  if (panel->priv->arrow && GTK_IS_WIDGET (panel->priv->arrow))
-    g_object_set (panel->priv->arrow, "active", FALSE, NULL);
+    if (!HILDON_DESKTOP_IS_PANEL_EXPANDABLE (panel))
+      return;
 
-  gtk_grab_remove (GTK_WIDGET (panel->priv->extension_window));
+    if (panel->priv->arrow && GTK_IS_WIDGET (panel->priv->arrow))
+      g_object_set (panel->priv->arrow, "active", FALSE, NULL);
 
-  gtk_widget_hide (GTK_WIDGET (panel->priv->extension_window));
+    gtk_grab_remove (GTK_WIDGET (panel->priv->extension_window));
 
-  panel->priv->extension_opened = FALSE;
+    gtk_widget_hide (GTK_WIDGET (panel->priv->extension_window));
+
+    panel->priv->extension_opened = FALSE;
+  }
 }
 
 static gboolean 
