@@ -61,8 +61,11 @@ desktop_window_changed (DesktopWindowData *data);
 struct _HildonDesktopPanelWindowCompositePrivate
 {
   Picture       background_picture;
-  Picture       background_mask;
-  gint          background_width, background_height;
+  guint         background_width, background_height;
+
+  Picture       pattern_picture;
+  Picture       pattern_mask;
+  guint         pattern_width, pattern_height;
 
   gint          x, y, width, height;
 
@@ -236,6 +239,90 @@ hildon_desktop_panel_window_composite_class_init (HildonDesktopPanelWindowCompos
 
 #ifdef HAVE_X_COMPOSITE
 
+static void
+hildon_desktop_panel_window_composite_update_background (HildonDesktopPanelWindowComposite *window,
+                                                         GdkRectangle *area)
+{
+  HildonDesktopPanelWindowCompositePrivate     *priv = window->priv;
+  HildonDesktopPanelWindowCompositeClass       *klass =
+      HILDON_DESKTOP_PANEL_WINDOW_COMPOSITE_GET_CLASS (window);
+  DesktopWindowData                            *data =
+        klass->desktop_window_data;
+
+  if (!priv->width || !priv->height) return;
+
+  if (priv->background_picture == None ||
+      priv->width != priv->background_width ||
+      priv->height != priv->background_height)
+  {
+    const GdkColor      white = { 0xFFFFFFFF, 0xFFFF, 0xFFFF, 0xFFFF };
+
+    if (priv->background_picture != None)
+      XRenderFreePicture (GDK_DISPLAY (), priv->background_picture);
+
+    priv->background_picture = hildon_desktop_picture_from_color (&white,
+                                                                  priv->width,
+                                                                  priv->height);
+
+    g_return_if_fail (priv->background_picture != None);
+
+    priv->background_width  = priv->width;
+    priv->background_height = priv->height;
+
+    area->x = 0;
+    area->y = 0;
+    area->width  = priv->width;
+    area->height = priv->height;
+  }
+
+  if (data->home_picture != None)
+
+  {
+    XRenderComposite (GDK_DISPLAY (),
+                      PictOpSrc,
+                      data->home_picture,
+                      None,
+                      priv->background_picture,
+                      priv->x + area->x, priv->y + area->y,
+                      priv->x + area->x, priv->y + area->y,
+                      area->x,
+                      area->y,
+                      area->width,
+                      area->height);
+  }
+
+  if (priv->pattern_picture != None)
+  {
+    if (priv->scale)
+    {
+      XRenderSetPictureTransform (GDK_DISPLAY (),
+                                  priv->pattern_picture,
+                                  &priv->transform);
+
+      if (priv->pattern_mask != None)
+        XRenderSetPictureTransform (GDK_DISPLAY (),
+                                    priv->pattern_mask,
+                                    &priv->transform);
+    }
+
+    XRenderComposite (GDK_DISPLAY (),
+                      PictOpOver,
+                      priv->pattern_picture,
+                      priv->pattern_mask,
+                      priv->background_picture,
+                      area->x, area->y,
+                      area->x, area->y,
+                      area->x, area->y,
+                      area->width, area->height);
+
+  }
+
+  gdk_window_invalidate_rect (GTK_WIDGET (window)->window,
+                              area,
+                              TRUE);
+
+}
+
 static GdkFilterReturn
 home_window_filter (GdkXEvent          *xevent,
                     GdkEvent           *event,
@@ -269,21 +356,27 @@ home_window_filter (GdkXEvent          *xevent,
         HildonDesktopPanelWindowComposite              *window = w->data;
         HildonDesktopPanelWindowCompositePrivate       *priv = window->priv;
 
-        if (priv->x + priv->width >= rects[i].x      &&
-            priv->x <= rects[i].x + rects[i].width   &&
-            priv->y + priv->height >= rects[i].y     &&
-            priv->y <= rects[i].y + rects[i].height)
+        if (priv->x + priv->width > rects[i].x      &&
+            priv->x < rects[i].x + rects[i].width   &&
+            priv->y + priv->height > rects[i].y     &&
+            priv->y < rects[i].y + rects[i].height)
 
         {
           GdkRectangle rect;
 
-          rect.x = rects[i].x;
-          rect.y = rects[i].y;
-          rect.width = rects[i].width;
-          rect.height = rects[i].height;
-          gdk_window_invalidate_rect (GTK_WIDGET (window)->window,
-                                      &rect,
-                                      TRUE);
+          /* Take the intersection, and offset it back
+           * to relate to the Panel window geometry */
+          rect.x = MAX (rects[i].x, priv->x) - priv->x;
+          rect.y = MAX (rects[i].y, priv->y) - priv->y;
+          rect.width =
+              MIN (rects[i].x + rects[i].width,  priv->x + priv->width) -
+              MAX (rects[i].x, priv->x);
+          rect.height =
+              MIN (rects[i].y + rects[i].height,  priv->y + priv->height) -
+              MAX (rects[i].y, priv->y);
+
+          hildon_desktop_panel_window_composite_update_background (window,
+                                                                   &rect);
         }
       }
     }
@@ -378,8 +471,8 @@ hildon_desktop_panel_window_composite_configure (GtkWidget             *widget,
   priv->width = event->width;
   priv->height = event->height;
 
-  if (priv->background_width != priv->width ||
-      priv->background_height != priv->height)
+  if (priv->pattern_width != priv->width ||
+      priv->pattern_height != priv->height)
   {
     XTransform scale = {{{ XDoubleToFixed ((gdouble)priv->background_width /priv->width), 0, 0},
                    {0, XDoubleToFixed ((gdouble)priv->background_height / priv->height), 0},
@@ -399,14 +492,19 @@ static gboolean
 hildon_desktop_panel_window_composite_expose (GtkWidget *widget,
                                               GdkEventExpose *event)
 {
+
+#if 0
+  g_debug ("Got expose event on %s, area %i,%i %ix%i",
+           gtk_widget_get_name (widget),
+           event->area.x,
+           event->area.y,
+           event->area.width,
+           event->area.height);
+#endif
   if (GTK_WIDGET_DRAWABLE (widget))
   {
-    HildonDesktopPanelWindowCompositeClass     *klass =
-        HILDON_DESKTOP_PANEL_WINDOW_COMPOSITE_GET_CLASS (widget);
     HildonDesktopPanelWindowCompositePrivate   *priv =
         HILDON_DESKTOP_PANEL_WINDOW_COMPOSITE (widget)->priv;
-    DesktopWindowData                          *data =
-        klass->desktop_window_data;
     GdkDrawable *drawable;
     gint x_offset, y_offset;
     Picture picture;
@@ -418,40 +516,17 @@ hildon_desktop_panel_window_composite_expose (GtkWidget *widget,
                                         &y_offset);
 
     picture = hildon_desktop_picture_from_drawable (drawable);
+    g_return_val_if_fail (picture, FALSE);
 
     g_object_set_data (G_OBJECT (drawable),
                        "picture", GINT_TO_POINTER (picture));
 
-    if (data->home_picture != None)
-      XRenderComposite (GDK_DISPLAY (),
-                        PictOpSrc,
-                        data->home_picture,
-                        None,
-                        picture,
-                        priv->x + event->area.x, priv->y + event->area.y,
-                        priv->x + event->area.x, priv->y + event->area.y,
-                        event->area.x - x_offset,
-                        event->area.y - y_offset,
-                        event->area.width,
-                        event->area.height);
-
     if (priv->background_picture != None)
     {
-      if (priv->scale)
-      {
-        XRenderSetPictureTransform (GDK_DISPLAY (),
-                                    priv->background_picture,
-                                    &priv->transform);
-
-        XRenderSetPictureTransform (GDK_DISPLAY (),
-                                    priv->background_mask,
-                                    &priv->transform);
-      }
-
       XRenderComposite (GDK_DISPLAY (),
-                        PictOpOver,
+                        PictOpSrc,
                         priv->background_picture,
-                        priv->background_mask,
+                        None,
                         picture,
                         event->area.x,
                         event->area.y,
@@ -497,6 +572,7 @@ static void
 hildon_desktop_panel_window_composite_unrealize (GtkWidget *widget)
 {
   HildonDesktopPanelWindowCompositeClass       *klass;
+
   GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 
   klass = HILDON_DESKTOP_PANEL_WINDOW_COMPOSITE_GET_CLASS (widget);
@@ -511,26 +587,28 @@ static void
 hildon_desktop_panel_window_composite_style_set (GtkWidget   *widget,
                                                  GtkStyle    *old_style)
 {
-  HildonDesktopPanelWindowCompositePrivate     *priv =
-      HILDON_DESKTOP_PANEL_WINDOW_COMPOSITE (widget)->priv;
+  HildonDesktopPanelWindowComposite            *window =
+      HILDON_DESKTOP_PANEL_WINDOW_COMPOSITE (widget);
+  HildonDesktopPanelWindowCompositePrivate     *priv = window->priv;
   const gchar                  *filename;
+  GdkRectangle                  rect = {0};
 
   if (!GTK_WIDGET_REALIZED (widget) ||
       !widget->style || !widget->style->rc_style)
     return;
 
-  if (priv->background_picture != None)
+  if (priv->pattern_picture != None)
     {
       XRenderFreePicture (GDK_DISPLAY (),
-                          priv->background_picture);
-      priv->background_picture = None;
+                          priv->pattern_picture);
+      priv->pattern_picture = None;
     }
 
-  if (priv->background_mask != None)
+  if (priv->pattern_mask != None)
     {
       XRenderFreePicture (GDK_DISPLAY (),
-                          priv->background_mask);
-      priv->background_mask = None;
+                          priv->pattern_mask);
+      priv->pattern_mask = None;
     }
 
   filename = widget->style->rc_style->bg_pixmap_name[GTK_STATE_NORMAL];
@@ -539,16 +617,16 @@ hildon_desktop_panel_window_composite_style_set (GtkWidget   *widget,
     return;
 
   hildon_desktop_picture_and_mask_from_file (filename,
-                                             &priv->background_picture,
-                                             &priv->background_mask,
-                                             &priv->background_width,
-                                             &priv->background_height);
+                                             &priv->pattern_picture,
+                                             &priv->pattern_mask,
+                                             &priv->pattern_width,
+                                             &priv->pattern_height);
 
-  if (priv->background_width != priv->width ||
-      priv->background_height != priv->height)
+  if (priv->pattern_width != priv->width ||
+      priv->pattern_height != priv->height)
   {
-    XTransform scale = {{{ XDoubleToFixed ((gdouble)priv->background_width /priv->width), 0, 0},
-                   {0, XDoubleToFixed ((gdouble)priv->background_height / priv->height), 0},
+    XTransform scale = {{{ XDoubleToFixed ((gdouble)priv->pattern_width /priv->width), 0, 0},
+                   {0, XDoubleToFixed ((gdouble)priv->pattern_height / priv->height), 0},
                    {0, 0, XDoubleToFixed (1.0)}}};
 
     priv->transform = scale;
@@ -557,6 +635,13 @@ hildon_desktop_panel_window_composite_style_set (GtkWidget   *widget,
   else
     priv->scale = FALSE;
 
+  rect.width = widget->allocation.width;
+  rect.height = widget->allocation.height;
+  hildon_desktop_panel_window_composite_update_background (window,
+                                                           &rect);
+
+  GTK_WIDGET_CLASS (parent_class)->style_set (widget, old_style);
+
 }
 
 #endif
