@@ -582,11 +582,11 @@ hd_wm_dbus_signal_handler (DBusConnection *conn, DBusMessage *msg, void *data)
 	return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
 
-    g_debug ("application starting: %s", service);
-
     app = hd_wm_lookup_application_via_service (service);
 
-    if (app)
+    if (app &&
+        !hd_wm_application_has_windows (app) &&
+        hd_wm_application_has_startup_notify (app))
     {
       g_signal_emit_by_name (hdwm,
                              "application-starting",
