@@ -108,6 +108,8 @@ struct _HDApplicationsMenuPrivate
   guint                     menu_update_timeout;
   gboolean                  focus_applications;
   gboolean                  motion_reset;
+
+  GList                    *apps_list;
 };
 
 static void hd_applications_menu_register_monitors (HDApplicationsMenu *button);
@@ -141,6 +143,9 @@ hd_applications_menu_finalize (GObject *gobject)
   if (TASKNAVIGATOR_ITEM (button)->menu)
     gtk_widget_destroy (GTK_WIDGET (TASKNAVIGATOR_ITEM (button)->menu));
   
+  if (button->priv->apps_list)
+    g_list_free (button->priv->apps_list);
+  
   G_OBJECT_CLASS (hd_applications_menu_parent_class)->finalize (gobject);
 }
 
@@ -246,6 +251,8 @@ hd_applications_menu_has_focus (HildonDesktopPopupMenu *menu)
       return TRUE;
   }
 
+  g_list_free (items);
+  
   return FALSE;
 }
 
@@ -377,32 +384,44 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
 {
   static GtkMenuItem *last_selected_item = NULL;
 
-  GList *sub_items, *i;
+  GList *sub_items = NULL;
   gboolean replaced = FALSE;
   
-  sub_items = hildon_desktop_popup_menu_get_children 
-	  (button->priv->menu_applications);
-  
+  if (GTK_IS_MENU_ITEM (last_selected_item))
+  {
+    sub_items = (GList *) g_object_get_data (G_OBJECT (last_selected_item),
+                                             CATEGORY_SUB_ITEMS);
+  }
+
   if (last_selected_item != item)
   {
-    last_selected_item = item;
+    GList *i;
 
-    replaced = TRUE;
-    
-    for (i = g_list_first (sub_items); i; i = i->next)
+    if (GTK_IS_MENU_ITEM (last_selected_item))
     {
-      GtkMenuItem *child = (GtkMenuItem *) i->data;
+      for (i = g_list_first (sub_items); i; i = i->next)
+      {
+        GtkWidget *child = (GtkWidget *) i->data;
 
-      g_object_ref (child);
+        gtk_widget_hide (child);
+      }
     }
 
-    g_list_free (sub_items);
-
+    last_selected_item = item;
+ 
+    replaced = TRUE;
+ 
     sub_items = (GList *) g_object_get_data (G_OBJECT (item),
                                              CATEGORY_SUB_ITEMS);
 
-    hildon_desktop_popup_menu_replace_items (button->priv->menu_applications, 
-		    		  	     sub_items);
+    for (i = g_list_first (sub_items); i; i = i->next)
+    {
+      GtkWidget *child = (GtkWidget *) i->data;
+
+      gtk_widget_show (child);
+    }
+
+    hildon_desktop_popup_menu_parent_size (button->priv->menu_applications);
   }
   else
   {
@@ -422,9 +441,6 @@ hd_applications_menu_activate_category (GtkMenuItem *item, HDApplicationsMenu *b
   }
 
   button->priv->focus_applications = FALSE;
-
-  if (!replaced)
-    g_list_free (sub_items);
 }
 
 static void
@@ -542,9 +558,8 @@ hd_applications_menu_activate_app (GtkMenuItem *item, HDApplicationsMenu *button
 }
 
 static void
-hd_applications_menu_free_menu_items (GList *menu_items)
+hd_applications_menu_free_menu_list (GList *menu_items)
 {
-  g_list_foreach (menu_items, (GFunc) gtk_widget_destroy, NULL);
   g_list_free (menu_items);
 }
 
@@ -632,7 +647,7 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
 
       gtk_container_add (GTK_CONTAINER (menu_item), hbox);
       
-      gtk_widget_show_all (hbox);
+      gtk_widget_show_all (menu_item);
 #endif
 
       GTK_WIDGET_SET_FLAGS (menu_item, GTK_CAN_FOCUS);
@@ -661,13 +676,15 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
 
         gtk_widget_set_sensitive (no_app_item, FALSE);
 	
+        button->priv->apps_list = g_list_prepend (button->priv->apps_list, no_app_item);
+
         sub_items = g_list_prepend (sub_items, no_app_item);
       }
 
       g_object_set_data_full (G_OBJECT (menu_item),
       		              CATEGORY_SUB_ITEMS,
       		              sub_items, 
-			      (GDestroyNotify) hd_applications_menu_free_menu_items);
+			      (GDestroyNotify) hd_applications_menu_free_menu_list);
 
       g_signal_connect (G_OBJECT (menu_item), 
 		        "activate",
@@ -743,9 +760,9 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
 			    TRUE, 
 			    0);
 
-	gtk_container_add (GTK_CONTAINER (menu_item), hbox);
-
 	gtk_widget_show_all (hbox);
+
+	gtk_container_add (GTK_CONTAINER (menu_item), hbox);
 #endif
       }
 
@@ -771,10 +788,16 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
       {
         GtkWidget *separator = gtk_separator_menu_item_new ();
 
-        if (level > 0)
+        if (level > 0) 
+	{
           gtk_widget_set_name (GTK_WIDGET (separator), HD_APPS_MENU_APP_MENU_ITEM_NAME);
+          button->priv->apps_list = g_list_prepend (button->priv->apps_list, separator);
+	}
         else
+        {
           gtk_widget_set_name (GTK_WIDGET (separator), HD_APPS_MENU_CAT_MENU_ITEM_NAME);
+	  gtk_widget_show (separator);
+        }
 
         menu_items = g_list_prepend (menu_items, separator);
       }
@@ -783,9 +806,12 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
         first_item = FALSE;
       }
 
+      if (level > 0)
+        button->priv->apps_list = g_list_prepend (button->priv->apps_list, menu_item);
+      
       menu_items = g_list_prepend (menu_items, menu_item);
     }
-      
+ 
     g_free (item_name);
     
     if (item_icon)
@@ -807,19 +833,20 @@ hd_applications_menu_get_items (HDApplicationsMenu *button,
 static void
 hd_applications_menu_populate (HDApplicationsMenu *button)
 {
-  GList *menu_items, *i;
+  GList *menu_items;
   GtkTreeIter iter;
+
+  g_list_free (button->priv->apps_list);
+  button->priv->apps_list = NULL;
   
   menu_items = hd_applications_menu_get_items (button, NULL, &iter, 0);
 
-  for (i = g_list_first (menu_items); i; i = i->next)
-  {
-    GtkMenuItem *menu_item = (GtkMenuItem *) i->data;
-
-    hildon_desktop_popup_menu_add_item
-      (button->priv->menu_categories, menu_item);
-  }
-
+  hildon_desktop_popup_menu_replace_items 
+    (button->priv->menu_categories, menu_items);
+  
+  hildon_desktop_popup_menu_replace_items 
+    (button->priv->menu_applications, button->priv->apps_list);
+  
   g_list_free (menu_items);
 }
 
