@@ -357,6 +357,14 @@ hd_switcher_menu_get_icon_from_theme (const gchar     *icon_name,
   return retval;
 }
 
+static gboolean  
+hd_switcher_menu_eventbox_keypress_cb (GtkWidget      *widget,
+			               GdkEventKey    *event,
+			               HDSwitcherMenu *switcher)
+{
+  return gtk_widget_event (GTK_BIN (widget)->child, (GdkEvent *) event);
+}
+
 static gboolean  
 hd_switcher_menu_popup_window_keypress_cb (GtkWidget      *widget,
 			                   GdkEventKey    *event,
@@ -724,6 +732,8 @@ hd_switcher_menu_add_notification_group (gpointer key,
     gtk_widget_set_name (GTK_WIDGET (menu_item), 
       		         HD_SWITCHER_MENU_NOT_MENU_ITEM_NAME);
 
+    gtk_widget_show (GTK_WIDGET (menu_item));
+    
     *items = g_list_append (*items, menu_item);
   }
 }
@@ -917,7 +927,7 @@ hd_switcher_menu_constructor (GType gtype,
   HDWM *hdwm = hd_wm_get_singleton ();
   const GtkWidget *button_up, *button_down;
   GdkPixbuf *pixbuf;
-  GtkWidget *button, *arrow;
+  GtkWidget *button, *arrow, *event_box;
 
   object = G_OBJECT_CLASS (hd_switcher_menu_parent_class)->constructor (gtype,
            	                                                        n_params,
@@ -972,26 +982,57 @@ hd_switcher_menu_constructor (GType gtype,
   gtk_widget_set_size_request (GTK_WIDGET (switcher->priv->notifications_window),
 			       AS_MENU_PANE_WIDTH, 1);  
 
+  event_box = gtk_event_box_new ();
+  
+  gtk_widget_set_name (event_box, 
+		       HD_SWITCHER_MENU_APP_MENU_NAME);
+
+  g_signal_connect (G_OBJECT (event_box),
+		    "key-press-event",
+		    G_CALLBACK (hd_switcher_menu_eventbox_keypress_cb),
+		    switcher);
+
+  gtk_widget_show (event_box);
+  
   switcher->priv->menu_applications =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  		    	     "item-height", 72,
 		  		    	     "parent", switcher->priv->popup_window,
 				    	     NULL));	    
   
+  gtk_widget_show (GTK_WIDGET (switcher->priv->menu_applications));
+
+  gtk_container_add (GTK_CONTAINER (event_box),
+		     GTK_WIDGET (switcher->priv->menu_applications));
+
+  gtk_container_add (GTK_CONTAINER (switcher->priv->popup_window),
+		     GTK_WIDGET (event_box));
+
+  event_box = gtk_event_box_new ();
+  
+  gtk_widget_set_name (event_box, 
+		       HD_SWITCHER_MENU_NOT_MENU_NAME);
+
+  g_signal_connect (G_OBJECT (event_box),
+		    "key-press-event",
+		    G_CALLBACK (hd_switcher_menu_eventbox_keypress_cb),
+		    switcher);
+
+  gtk_widget_show (event_box);
+  
   switcher->priv->menu_notifications =
     HILDON_DESKTOP_POPUP_MENU (g_object_new (HILDON_DESKTOP_TYPE_POPUP_MENU,
 		  			     "item-height", 72,
 		  		    	     "parent", switcher->priv->notifications_window,
 					     NULL));
 
-  gtk_container_add (GTK_CONTAINER (switcher->priv->popup_window),
-		     GTK_WIDGET (switcher->priv->menu_applications));
+  gtk_widget_show (GTK_WIDGET (switcher->priv->menu_notifications));
 
-  gtk_container_add (GTK_CONTAINER (switcher->priv->notifications_window),
+  gtk_container_add (GTK_CONTAINER (event_box),
 		     GTK_WIDGET (switcher->priv->menu_notifications));
 
-  gtk_widget_show (GTK_WIDGET (switcher->priv->menu_applications));
-  gtk_widget_show (GTK_WIDGET (switcher->priv->menu_notifications));
+  gtk_container_add (GTK_CONTAINER (switcher->priv->notifications_window),
+		     GTK_WIDGET (event_box));
 
   button_up = 
     hildon_desktop_popup_menu_get_scroll_button_up (switcher->priv->menu_applications);
@@ -1148,6 +1189,8 @@ hd_switcher_menu_constructor (GType gtype,
                           G_CALLBACK (hd_switcher_menu_item_activated),
                           switcher);
 
+  gtk_widget_show (GTK_WIDGET (switcher->priv->home_item));
+  
   hd_switcher_menu_create_applications_menu (switcher, hdwm);
   
   switcher->priv->notification_groups = 
@@ -1823,6 +1866,8 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
           		        G_CALLBACK (hd_switcher_menu_item_activated),
           		        switcher);
 
+        gtk_widget_show (GTK_WIDGET (menu_item));
+	
 	children = g_list_append (children, menu_item);
 
         if (!ack && !icon_is_set)
@@ -1909,6 +1954,8 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
 			      G_CALLBACK (hd_switcher_menu_item_activated),
 			      (gpointer)switcher);
 
+      gtk_widget_show (GTK_WIDGET (menu_item));
+      
       items = g_list_append (items, menu_item);
 
       if (hd_wm_entry_info_is_active (entry))
