@@ -266,7 +266,7 @@ app_button_toggled_cb (GtkToggleButton *toggle,
   gboolean is_active = gtk_toggle_button_get_active (toggle);
   gboolean is_inconsistent = gtk_toggle_button_get_inconsistent (toggle);
   gchar *name,*name_pressed;
-  
+
   name =
    g_strdup_printf (AS_BUTTON_NAME,pos+1);
 
@@ -466,15 +466,23 @@ hn_app_switcher_init_sound_samples (HNAppSwitcher *app_switcher)
  * -- currently we are only interested in the system inactivity event
  */
 static
-void hn_app_osso_hw_cb (osso_hw_state_t *state, gpointer data)
+void hn_app_osso_display_state_cb (osso_display_state_t state, gpointer data)
 {
-  g_return_if_fail(state && data);
+  gboolean system_inactivity;
+
+  g_return_if_fail(data);
+
   HNAppSwitcher *app_switcher = HN_APP_SWITCHER (data);
   HNAppSwitcherPrivate *priv = app_switcher->priv;
 
-  if (state->system_inactivity_ind != priv->system_inactivity)
+  if (state == OSSO_DISPLAY_DIMMED)
+    return;
+
+  system_inactivity = (state == OSSO_DISPLAY_OFF); 
+  
+  if (system_inactivity != priv->system_inactivity)
   {
-    priv->system_inactivity = state->system_inactivity_ind;
+    priv->system_inactivity = system_inactivity;
     queue_refresh_buttons (app_switcher);
   }
 }
@@ -483,7 +491,6 @@ static void
 hn_app_switcher_osso_initialize (HNAppSwitcher *app_switcher)
 {
   HNAppSwitcherPrivate *priv = app_switcher->priv;
-  osso_hw_state_t hs = {0};
   
   priv->osso = osso_initialize ("AS_DIMMED_infoprint", "0.1", FALSE, NULL);
 
@@ -495,9 +502,9 @@ hn_app_switcher_osso_initialize (HNAppSwitcher *app_switcher)
   }
 
   /* register stystem inactivity handler */
-  hs.system_inactivity_ind = TRUE;
-  osso_hw_set_event_cb(priv->osso, &hs,
-                       hn_app_osso_hw_cb, app_switcher);
+  osso_hw_set_display_event_cb(priv->osso,
+                               hn_app_osso_display_state_cb, 
+			       app_switcher);
   
   
 }
